﻿------------------------------------------------------------------------------
--	GridStatusEmphasize by Slaren
------------------------------------------------------------------------------

local QuickHealth = LibStub and LibStub("LibQuickHealth-2.0", true) -- don't error if not found
local GridStatus = Grid:GetModule("GridStatus")

GridStatusEmphasize = GridStatus:NewModule("GridStatusEmphasize", "AceTimer-3.0")
GridStatusEmphasize.menuName = "Emphasize"

-- acetimer
--LibStub():Embed(GridStatusMRDS)

-- module
GridStatusEmphasize.modulePrototype = {
	Debug = Grid.Debug,
}

function GridStatusEmphasize.modulePrototype:OnInitialize()
	if not self.db then
		-- GridStatus.core:RegisterDefaults(self.name, "profile", self.defaultDB or {})
		-- self.db = GridStatus.core:AcquireDBNamespace(self.name)
		self.db = Grid.db:RegisterNamespace(self.name, { profile = self.defaultDB or { } })		
	end
	
	self.debugging = GridStatusEmphasize.db.profile.debug
	self.debugFrame = self.debugFrame
end

function GridStatusEmphasize.modulePrototype:RegisterOptions(name, args)
	GridStatusEmphasize.emphasize_options[name] = {
		type ="group",
		name = name,
		desc = "Options for " .. name,
		args = args,
	}
	
	self.options = args
end

function GridStatusEmphasize:OnModuleCreated(module)
	module.super = self.modulePrototype
end


GridStatusEmphasize:SetDefaultModulePrototype(GridStatusEmphasize.modulePrototype)
-- GridStatusEmphasize:SetDefaultModuleLibraries("AceEvent-3.0") -- SetModuleMixins("AceDebug-2.0")

-- upvalues
local GridRoster = Grid:GetModule("GridRoster")
local GridFrame = Grid:GetModule("GridFrame")
local next = next
local settings

-- data
-- [guid] = { [tag] = true, [tag2] = true.. }
local temp_emphasized_units = {}
local static_emphasized_units = {}
local is_paused = false
local is_enabled = false


GridStatusEmphasize.defaultDB = {
	debug = false,
	is_first = true,

	alert_emphasize = {
		enable = true,
		priority = 99,
		
		others_alpha = 0.1,
	}
}

local options = {
	color = false,
	range = false,
	["others_alpha"] = {
		order = 100,
		type = "range",
		name = "Alpha for faded out units",
		desc = "Alpha for faded out units",
		get = function () return GridStatusEmphasize.db.profile.alert_emphasize.others_alpha end,
		set = function (_, v) GridStatusEmphasize.db.profile.alert_emphasize.others_alpha = v end,
		min = 0,
		max = 1,
		step = 0.01,
	},
	["fns"] = {
		order = 101,
		type = "execute",
		name = "Test Flash",
		desc = "Test flash",
		func = function() GridStatusEmphasize:Flash() end,
	},
}

function GridStatusEmphasize:OnInitialize()
	self.super.OnInitialize(self)
	
	self:RegisterStatus("alert_emphasize", "Emphasize", options, true)
	
	self.emphasize_options = GridStatus.options.args["alert_emphasize"].args
	
	settings = self.db.profile.alert_emphasize
	
	-- set emphasize to frame alpha on first run
	if self.db.profile.is_first then
		GridFrame.db.profile.statusmap["frameAlpha"]["alert_emphasize"] = true
		GridFrame:UpdateAllFrames()
		self.db.profile.is_first = false
	end	
end

function GridStatusEmphasize:OnStatusEnable(status)
	if status == "alert_emphasize" then
		is_enabled = true
		self:RegisterEvent("Grid_RosterUpdated", "RosterUpdated")
	end
end

function GridStatusEmphasize:OnStatusDisable(status)
	if status == "alert_emphasize" then
		is_enabled = false
		self:UnregisterEvent("Grid_RosterUpdated")
		self.core:SendStatusLostAllUnits("alert_emphasize")
	end
end

local function tisempty(t)
	return next(t) == nil
end

function GridStatusEmphasize:RosterUpdated()
	local need_update = false

	for guid, tags in pairs(temp_emphasized_units) do
		if not GridRoster:IsGUIDInRaid(guid) then
			temp_emphasized_units[guid] = nil
			need_update = true
		end
	end

	for guid, tags in pairs(static_emphasized_units) do
		if not GridRoster:IsGUIDInRaid(guid) then
			static_emphasized_units[guid] = nil
			need_update = true
		end
	end
	
	if need_update then
		self:Update()
	end
end

function GridStatusEmphasize:Update()
	if not is_enabled or is_paused then
		return
	end
	
	if self.db.profile.debug and not tisempty(temp_emphasized_units) then
		local debugstr = "Temporary: "
		for guid, tags in pairs(temp_emphasized_units) do
			debugstr = debugstr .. (UnitName(GridRoster:GetUnitidByGUID(guid)) or "nil") .. "[ "			
			for tag, v in pairs(tags) do debugstr = debugstr .. tag .. " " end
			debugstr = debugstr .. "] "
		end
		self:Debug(debugstr)

		local debugstr = "Static: "
		for guid, tags in pairs(static_emphasized_units) do
			debugstr = debugstr .. (UnitName(GridRoster:GetUnitidByGUID(guid)) or "nil") .. "[ "			
			for tag, v in pairs(tags) do debugstr = debugstr .. tag .. " " end
			debugstr = debugstr .. "] "
		end
		self:Debug(debugstr)
	end

	if tisempty(temp_emphasized_units) then
		self.core:SendStatusLostAllUnits("alert_emphasize")
	else
		for guid, unitid in GridRoster:IterateRoster() do
			if temp_emphasized_units[guid] or static_emphasized_units[guid] then
				self.core:SendStatusLost(guid, "alert_emphasize")
			else
				self.core:SendStatusGained(guid,
							"alert_emphasize",
							settings.priority,
							nil,
							{r=0, g=0, b=0, a=settings.others_alpha},
							1,
							nil,
							nil)
			end
		end
	end
end

 function GridStatusEmphasize:AddEmphasize(t, guid, tag)
	-- self:Debug("Add ", tag)
	if not t[guid] then
		t[guid] = { [tag] = true }
		self:Update()
	else
		t[guid][tag] = true
	end
end

function GridStatusEmphasize:RemoveEmphasize(t, guid, tag)
	-- self:Debug("Remove ", tag)
	if t[guid] then
		t[guid][tag] = nil
		if tisempty(t[guid]) then
			t[guid] = nil
			self:Update()
		end
	end
end

function GridStatusEmphasize:RemoveAllEmphasizes(t, tag)
	-- self:Debug("RemoveAll ", tag)
	local need_update = false
	for guid, _ in pairs(t) do
		if t[guid][tag] then
			t[guid][tag] = nil
			if tisempty(t[guid]) then
				t[guid] = nil
				need_update = true
			end
		end
	end
	
	if need_update then	
		self:Update()
	end
end

-- API
-- Static Emphasizes: won't trigger a emphasize, but they will remain visible while emphasizing
function GridStatusEmphasize:StaticEmphasizeUnit(guid, tag)
	self:AddEmphasize(static_emphasized_units, guid, tag)
end

function GridStatusEmphasize:StaticDeemphasizeUnit(guid, tag)
	self:RemoveEmphasize(static_emphasized_units, guid, tag)
end

function GridStatusEmphasize:StaticDeemphasizeAllUnits(tag)
	self:RemoveAllEmphasizes(static_emphasized_units, tag)
end

-- Temp Emphasizes: these actually will start and stop emphasizes
function GridStatusEmphasize:TempEmphasizeUnit(guid, tag)
	self:AddEmphasize(temp_emphasized_units, guid, tag)
end

function GridStatusEmphasize:TempDeemphasizeUnit(guid, tag)
	self:RemoveEmphasize(temp_emphasized_units, guid, tag)
end

function GridStatusEmphasize:TempDeemphasizeAllUnits(tag)
	self:RemoveAllEmphasizes(temp_emphasized_units, tag)
end

-- Pause and resume updating
function GridStatusEmphasize:Pause()
	is_paused = true
end

function GridStatusEmphasize:Resume()
	is_paused = false
	self:Update()
end

-- Flash
local fns_step
local fns_num_step
function GridStatusEmphasize:Flash(steps)
	if not is_enabled then
		return
	end

	self:CancelTimer(self.flash_timer, true)
	self:Pause()

	fns_step = 1
	fns_num_step = steps or 2
	self.flash_timer = self:ScheduleRepeatingTimer("Flash_Step", 0.200)
	self:Flash_Step()
end

function GridStatusEmphasize:FlashStop()
	self:CancelTimer(self.flash_timer, true)
	self:Resume()
end

function GridStatusEmphasize:Flash_Step()
	if (fns_step % 2) == 0 then
		-- show
		self.core:SendStatusLostAllUnits("alert_emphasize")
	else
		-- hide
		for guid, unitid in GridRoster:IterateRoster() do
			self.core:SendStatusGained(guid,
						"alert_emphasize",
						settings.priority,
						nil,
						{r=0, g=0, b=0, a=settings.others_alpha},
						1,
						nil,
						nil)
		end
	end
	
	fns_step = fns_step + 1
	if fns_step == (fns_num_step*2+1) then
		self:FlashStop()
	end
end
